'A note on sample handling.   A lot of the routines in this file change the sample in order to perform their calculations.  This is problematic in two ways.  The first is that they should not trash the workfile sample - i.e. we'd like the workfile sample to be reset to whatever it was on entering the routine.  This is easily handled with the "local smpl" command at the beginning of the sample.  The second problem is that often we want to set the sample inside the routine, but have it based on the incoming sample.  For example we might want to set the sample to be the second observation of the incoming sample onwards.  This is achieved in a slightly cumbersome, but easy, manner.  First we create a series, called smplser that is equal to 1.  Since we haven't changed the sample yet, this series will only be equal to 1 for observations that are in the incoming sample.  Any other observations will have a value of NA.   We can then use smplser to do some arithmatic in setting the samples inside the routines.  The best arithmatic is done with the @cumsum function.  @cumsum(smplser) will return the in-sample observation number of each observation in the incoming sample.  We can then set samples based on @cumsum(smplser). 
'Common uses of this are  (note the series cumser has already been set equal to @cumsum(smplser):
'Find out how many observations are in the current sample -    !totalobs = @max(cumser)                    this could have been done more easily with the @obssmpl command, but I leave this new form for instructional purposes.
'set the sample to be the second observation of the incoming sample up until the last observation of the incoming sample, not including any non-included observations inbetween:        smpl if cumser>1 and cumser<!totalobs+1 and smplser=1



subroutine local accumdist(series out, series high, series low, series close, series volume)
	local smpl
	series smplser=1
	smpl if (high-low)<> 0 and smplser=1
	out = (((close-low) - (high-close)) / (high-low))*volume
	out = @cumsum(out)
endsub

subroutine local accumswing(series out, series high, series low, series open, series close, scalar t)
	series swing
	call swing(swing,high,low,open,close,t)
	out = @cumsum(swing)
endsub

subroutine local avtr(series out, series high, series low, series close, scalar n)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	series hml = high-low
	series hmpc = high - close(-1)
	series pcml = close(-1) - low
	group g hml hmpc pcml
	series tr = @rmax(g)
	sample s  if cumser>0 and cumser < {n}+1 and smplser=1
	series atr = @mean(tr,s)
	smpl if cumser >{n} and cumser <!totalobs+1 and smplser=1
	atr = (tr-atr(-1))/14 + atr(-1)
	out = atr
endsub	

subroutine bollinger(series out, series lowout, series highout, series price, scalar n, scalar k)
	out = @movav(price,n)
	lowout = out - k*@movstdev(price,n)
	highout = out + k*@movstdev(price,n)
endsub


subroutine local chaikinflow(series out, series high, series low, series close, series volume, scalar n)
	series accdist
	call accumdist(accdist,high,low,close,volume)
	out = @movsum(accdist,n) / @movsum(volume,n)
endsub

subroutine local chaikinosc(series out, series high, series low, series close, series volume)
	series accdist
	call accumdist(accdist,high,low,close,volume)
	series sum = @cumsum(accdist)
	series ema3
	series ema10
	call ema(ema3,sum,3)
	call ema(ema10,sum,10)
	out = ema3 - ema10
endsub

subroutine local chande(series out, series close, scalar n)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	series su = close - close(-1)
	su = @recode(su>0,su,na)
	series sd = close - close(-1)
	sd = @recode(sd<0,abs(sd),na)
	smpl if cumser > {n} and cumser<!totalobs+1 and smplser=1
	series msu = @msum(su,n)
	series msd = @msum(sd,n)
	out = (msu-msd)/(msu+msd)*100
endsub

subroutine local cci(series out, series high, series low, series close, scalar n)
	group g high low close
	series mean = @rmean(g)
	series mmean = @movav(mean,n)
	series gcol = abs(mmean-mean) 
	for !i=1 to n-1
		gcol = gcol + abs(mmean-mean(-!i))
	next
	series out = gcol/5*0.015
	out = (mean-mmean)/out
endsub

' Scalar inputs are periods, "value of a 1cent move", margin requirement, commission ($).
subroutine local csi(series out, series high, series low, series open, series close, scalar p, scalar v, scalar m, scalar c)
	!k = v/@sqrt(m)*1/(150+c)*100
	series temp1
	series temp2
	series temp3
	series temp4
	series temp5
	call directmov(temp1,temp2,temp3,temp4,temp5,out,high,low,open,close,p,v,m,c)
endsub

'outputs are +di, -di, dx, adx, adxr, csi.     ' Scalar inputs are periods, "value of a 1cent move", margin requirement, commission ($).
subroutine directmov(series outpdi, series outmdi, series outdx, series outadx, series outadxr, series outcsi, series high, series low, series open, series close, scalar p, scalar v, scalar m, scalar c)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	group g high-low high-close(-1) close(-1)-low
	smpl if cumser>1 and cumser<!totalobs+1 and smplser=1
		series tr = @rmax(g)
	smpl @all if smplser=1
	series pdm1=0
	series mdm1=0
	smpl if (high-high(-1)) > (low(-1)-low) and high>high(-1) and smplser=1
		pdm1 = high-high(-1)
	smpl if (low(-1)-low) > (high-high(-1)) and low<low(-1) and smplser=1
		mdm1 = low(-1)-low
	smpl if cumser>smplser=1
	series trp = @msum(tr,p)
	series mdmp = @msum(mdm1,p)
	series pdmp = @msum(pdm1,p)
	smpl if cumser > {p} and cumser<!totalobs+1 and smplser=1
		trp = trp(-1) - (trp(-1)/p) + tr
		mdmp = mdmp(-1) - (mdmp(-1)/14) + mdm1
		pdmp = pdmp(-1) - (pdmp(-1)/14) + pdm1
		outpdi = @round(pdmp/trp*100)
		outmdi = @round(mdmp/trp*100)
		series didiff = abs(outpdi-outmdi)
		series disum = outpdi+outmdi
		outdx = @round(didiff/disum*100)
		outadx = @round(@movav(outdx,p))
		series atrp = trp/p
	smpl if cumser>2*{p} and cumser<!totalobs+1 and smplser=1
		outadx = @round((outadx(-1)*(p-1)+outdx)/p)
		outadxr = @round((outadx+outadx(-(p-1)))/2)
		!k = v/@sqrt(m)*1/(150+c)*100
		outcsi = outadxr * atrp * !k		
endsub

subroutine local dema(series out, series in, scalar n)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	series ema
	call ema(ema,in,n)
	series ema2
	smpl if cumser>n-1 and cumser<!totalobs+1 and smplser=1
	call ema(ema2,ema,n)
	smpl if cumser>2*n-2 and cumser<!totalobs+1 and smplser=1
	out = (2*ema)-ema2
endsub

subroutine local dmi(series out, series in)
	series vindex = @movstdevp(in,5) / @movav(@movstdevp(in,5),10)
	series days = @floor(14/vindex)
	days = @recode(days>30,30,days)
	days = @recode(days<3,3,days)
	series temp
	for !i=1 to @obsrange
		!n = days(!i)
		if !n>0 then
			call rsi(temp,in,!n)
			out(!i) = temp(!i)
		endif
	next
endsub
	
subroutine local dpo(series out, series close, scalar n)
	!n2 = @floor((n/2)+1)
	series temp = @movav(close,n)
	out = close - temp(!n2)
endsub

subroutine local emv(series out, series high, series low, series volume, scalar n)
	series mm = (high+low)/2 - (high(-1)+low(-1))/2
	series boxr = (volume/10000)/(high-low)
	series emv = mm/boxr
	out = @movav(emv,n)
endsub

subroutine local ema(series out, series in, scalar n)
	local smpl
	series s = 1
	out = in
	!p = 2/(n+1)
	smpl if @cumsum(s)>1   and s = 1'this trick lets me set the sample to the 2nd observation in the current sample.
	out = !p*in + (1-!p)*out(-1)
endsub

subroutine local imi(series out, series open, series close, scalar n)
	series isu = @recode(close>open,close-open,0)
	series isd = @recode(close<open,open-close,0)
	series misu = @movsum(isu,n)
	series misd = @movsum(isd,n)
	out = misu/(misu+misd)*100
endsub

subroutine local klinger(series out, series high, series low, series close, series volume)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)	
	series trend  = 0
	series cummeas =0
	smpl if cumser>1 and cumser<!totalobs+1 and smplser=1
	trend = @recode(high+low+close>high(-1)+low(-1)+close(-1),1,-1)
	cummeas = @recode(trend=trend(-1), cummeas(-1)+high-low,high-low+high(-1)-low(-1))
	series volfor = volume*(abs(2*((high-low)/cummeas)-1))*trend*100
	series ema34
	call ema(ema34,volfor,34)
	series ema55
	call ema(ema55,volfor,55)
	out = ema34-ema55
endsub

subroutine local macd(series out, series in)
	%s = @pagesmpl
	smpl @all
	series ema15 = in
	series ema75=in
	smpl @first+1 @last
	ema15 = 0.15*in + 0.85*ema15(-1)
	ema75 = 0.075*in + 0.925*ema75(-1)
	out = ema15-ema75
	smpl {%s}
endsub

subroutine local mfi(series out, series high, series low, series volume)
	out = (high-low)/volume
endsub

subroutine local massidx(series out, series high, series low, scalar n)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	series ema
	series hl = high-low
	call ema(ema,hl,9)
	series ema2
	smpl if cumser>8 and cumser<!totalobs+1 and smplser=1
	call ema(ema2,ema,9)
	smpl if cumser>16 and cumser<!totalobs+1 and smplser=1
	series div = ema/ema2
	out = @movsum(div,n)
endsub

subroutine local monfi(series out, series high, series low, series close, series volume, scalar n)
	local smpl
	series price = (high+low+close)/3 
	series priceup = @recode(price>price(-1),price*volume,0)
	series pricedown = @recode(price<price(-1),price*volume,0)
	series sumup = @movsum(priceup,n)
	series sumdwn = @movsum(pricedown,n)
	smpl if sumup/sumdwn >0
	out = 100 - 100/(sumup/sumdwn+1)
endsub

'subroutine the calculates different moving averages.  Type can be "Simp", "Exp" (exponential), "Tri" (Triangular", "Weight" (Weighted)
subroutine local movav(series out,series in, scalar n, string type)
	%type = @upper(type)
	if %type="SIMP" then
		out = @movav(in,n)
	endif
	if %type = "EXP" then
		call ema(out,in,n)
	endif
	if %type = "TRI" then
		out = @movav(@movav(in,n),n)
	endif
	if %type = "WEIGHT" then
		out = in*n
		for !ii = 1 to n
			out = out + in(-!ii)*(n-!ii)
		next
		!div = n*(n+1)/2
		out = out / !div
	endif
endsub

'routine that calculates a moving slope cofficient where slope is defined as the B parameter in Y = a + BT, and T is a time trend.
subroutine movtrendslope(series out, series in, scalar n)
		out = @movcov(in,@trend+1,n) / @movvar(@trend+1,n)
endsub

subroutine nvi(series out, series close, series volume)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	out = 1000
	smpl if cumser>1 and cumser<!totalobs+1 and smplser=1
			out = @recode(volume<volume(-1),((close-close(-1))/close(-1))*out(-1)+out(-1),out(-1))
endsub

subroutine obv(series out, series close, series volume)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	out = 0
	smpl if cumser>1 and cumser<!totalobs+1 and smplser=1
		out = @recode(close>close(-1),out(-1)+volume, @recode(close<close(-1),out(-1)-volume,out(-1)))
endsub

subroutine perform(series out, series in)
	!first = in(1)
	out = 100*(in-!first)/!first
endsub

subroutine priceosc(series out, series in, scalar n1, scalar n2)
	out = 100*(@movav(in,n1)-@movav(in,n2))/@movav(in,n2)
endsub

subroutine local projosc(series out, series high, series low, series close, scalar n)
	series highslope
	call movtrendslope(highslope,high,n)
	series lowslope
	call movtrendslope(lowslope,low,n)

	'build up groups of 1 to N members
	group ghigh high
	group glow low
	for !ii = 1 to n-1
		ghigh.add high(-!ii)+!ii*highslope
		glow.add low(-!ii)+!ii*lowslope
	next
	series upband = @rmax(ghigh)
	series lowband = @rmin(glow)
	
	out = 100*(close-lowband)/(upband-lowband)
endsub

subroutine pvi(series out, series close, series volume)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	out = 1000
	smpl if cumser>1 and cumser<!totalobs+1 and smplser=1
		out = @recode(volume>volume(-1),out(-1) + out(-1)*(close-close(-1))/close(-1), out(-1))
endsub

subroutine pvt(series out, series close, series volume)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	out=0
	smpl if cumser>1 and cumser<!totalobs+1 and smplser=1
		out = volume*(close-close(-1))/close(-1)+out(-1)
endsub

subroutine qstick(series out, series open, series close, scalar n)
	out = (@movsum(close-open,n))/n
endsub

subroutine local ri(series out, series high, series low, series close, series volume, scalar q, scalar m)
		local smpl
		series smplser = 1
		series cumser = @cumsum(smplser)
		!totalobs = @max(cumser)
		group gg high-low high-close(-1) close(-1)-low
		series trange = @rmax(gg)
		series g = @recode(close>close(-1),trange/(close-close(-1)),trange)
		series min = @movmin(g,q)
		series max = @movmax(g,q)
		series j = @recode(max>min,100*(g-min)/(max-min),100*(g-min))
		smpl if cumser>{q} and cumser<!totalobs+1 and smplser=1
		call ema(out,j,m)
endsub

subroutine local rsi(series out, series in, scalar n)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	series up = @recode(in>in(-1),in-in(-1),0)
	series upsmooth = @movav(up,n)
	series down = @recode(in<in(-1),in(-1)-in,0)
	series downsmooth = @movav(down,n)
	smpl if cumser>{n}+1 and cumser<!totalobs+1 and smplser=1
	upsmooth = upsmooth(-1)+(up-upsmooth(-1))/n
	downsmooth = downsmooth(-1) + (down-downsmooth(-1))/n
	smpl if smplser=1
	out = 100 - 100/(upsmooth/downsmooth + 1)
endsub

subroutine local smi(series out, series high, series low, series close, scalar n, scalar p)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	smpl if cumser>{n}-1 and cumser<!totalobs+1 and smplser=1
		series mid = close - (@mmax(high,n) + @mmin(low,n))/2
		series diff = @mmax(high,n) - @mmin(low,n)
		series smoothm1
		series smoothd1
		call ema(smoothm1,mid,p)
		call ema(smoothd1,diff,p)
	smpl if cumser> {n}-1+{p}-1 and cumser<!totalobs+1 and smplser=1
		series smoothm2
		series smoothd2
		call ema(smoothm2,smoothm1,p)
		call ema(smoothd2,smoothd1,p)
		out = smoothm2/smoothd2*200
endsub

subroutine local stochosc(series out, series high, series low, series close, scalar kp, scalar ks, scalar n, string movavtype)
	series dlow = close - @mmin(low,kp)
	series dhigh = @mmax(high,kp) - @mmin(low,kp)
	series sum = 100 * @msum(dlow,ks) / @msum(dhigh,ks)
	call movav(out,sum,n,movavtype)
endsub


'Currently all calculations match those in the accompanying spreadsheet to "Technical Analysis from A to Z".  There appear to be a number of discrepancies between the spreadsheet and the printed formulas.
subroutine local swing(series out, series high, series low, series open, series close, scalar t)
	local smpl
	series smplser=1

'Seems to be some disagreement on the calculation of K.  It is either the max of (High-close(-1)) and (Low-close(-1)), or the max of (High(-1)-close) and (Low(-1)-close).   
'	group g abs(high(-1)-close) abs(low(-1)-close)
	group gg abs(high-close(-1)) abs(low-close(-1))
	series k = @rmax(gg)

	series r
	series hc = abs(high-close(-1))
	series lc = abs(low-close(-1))
	smpl if (hc>lc) and (hc>abs(high-low)) and smplser=1
		r = hc - 0.5*lc + 0.25*abs(close(-1)-open(-1))
	smpl if (lc>hc) and (lc>abs(high-low)) and smplser=1
		r = lc - 0.5*hc + 0.25*abs(close(-1)-open(-1))
	smpl if (abs(high-low)>hc) and (abs(high-low)>lc) and smplser=1
		r = abs(high-low) + 0.25*abs(close(-1)-open(-1))
	smpl if smplser=1
	
	series n = (close-close(-1)) + 0.5*(close-open) + 0.25*(close(-1)-open(-1))
	
	out = 50 * n/r * k/t
	out = @recode(out>100,100,out)
endsub

subroutine local tema(series out, series in, scalar n)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	series ema
	call ema(ema,in,n)
	series ema2
	smpl if cumser>{n}-1 and cumser<!totalobs+1 and smplser=1
		call ema(ema2,ema,n)
		series ema3
		smpl if cumser>2*({n}-1) and cumser<!totalobs+1 and smplser=1
			call ema(ema3,ema2,n)
			smpl if cumser>3*({n}-1) and cumser<!totalobs+1 and smplser=1
				out = (3*ema)-3*ema2+ema3
endsub

subroutine local trix(series out, series close, scalar n)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	series ema1
	call ema(ema1,close,n)
	smpl if cumser>{n}-1 and cumser<!totalobs+1 and smplser=1
		series ema2
		call ema(ema2,ema1,n)
		smpl if cumser>2*({n}-1) and cumser<!totalobs+1 and smplser=1
			series ema3
			call ema(ema3,ema2,n)
			smpl if cumser>3*({n}-1)+1 and cumser<!totalobs+1 and smplser=1
				out = 100*(ema3-ema3(-1))/ema3(-1)
endsub

subroutine local ultosc(series out, series high, series low, series close)
	local smpl
	series smplser = 1
	series range =  @recode(high>close(-1),high,close(-1)) - @recode(low<close(-1),low,close(-1))
	series buyunits = close - @recode(low<close(-1),low,close(-1))
	series range7 = @movsum(range,7)
	series buyunits7 = @movsum(buyunits,7)
	smpl if range7<>0 and smplser=1
		series div7 = buyunits7/range7
	smpl if smplser=1
	series range14 = @movsum(range,14)
	series buyunits14 = @movsum(buyunits,14)
	smpl if range14<>0 and smplser=1
		series div14 = buyunits14/range14
	smpl if smplser=1
	series range28 = @movsum(range,28)
	series buyunits28 = @movsum(buyunits,28)
	smpl if range28<>0 and smplser=1
		series div28 = buyunits28/range28
	smpl if smplser=1
	out = (4*div7+2*div14+div28)/7*100
endsub

subroutine local chaikvol(series out, series high, series low, scalar n)
	series ema
	series temp1 = high-low
	call ema(ema,temp1,n)
	series temp2 = ema - ema(-n)
	out = temp2/ema(-n)*100
endsub

subroutine local volosc(series out, series volume, scalar n1, scalar n2)
	series msum2 = @movav(volume,n1)
	series msum5 = @movav(volume,n2)
	out = 100*(msum2-msum5)/msum5
endsub

subroutine local wildsmooth(series out, series close, scalar n)
	local smpl
	series smplser = 1
	series cumser = @cumsum(smplser)
	!totalobs = @max(cumser)
	series mav = @movav(close,n)
	smpl if cumser>{n} and cumser<!totalobs+1 and smplser=1
	mav = (close-mav(-1))/5 + mav(-1)
	out = mav
endsub

subroutine local willaccdist(series out, series high, series low, series close)
	series th = @recode(close(-1)>high,close(-1),high)
	series tl = @recode(close(-1)<low,close(-1),low)
	series ad = @recode(close>close(-1),close-tl,close-th)
	ad = @recode(close=close(-1),0,ad)
	out= @cumsum(ad)
endsub

subroutine local willr(series out, series high, series low, series close, scalar n)
	series mhigh = @movmax(high,n)
	series mlow = @movmin(low,n)
	out = (mhigh-close)/(mhigh-mlow)*(-100)
endsub
